
Stage = newClass("Stage")
Stage.SCALE = 2.0/20.0 -- 2 meters for 20 pixels
Stage.PLAYER_COUNT = 2
Stage.DAMAGE_COLOR = love.graphics.newColor(196, 0, 0)
Stage.PLAYER_COLORS = { 
						love.graphics.newColor(200, 60, 60),
						love.graphics.newColor(60, 200, 0),
						love.graphics.newColor(0, 80, 255),
						love.graphics.newColor(255, 200, 0),
					}

function Stage:init()
	self.finished = false
	self.winner = 0
	self.tipImage = Resource:getImage("data/tip.png")
	-- Create the physical world
	self.world = love.physics.newWorld(2 * 800 * self.SCALE, 2 * 600 * self.SCALE)
	self.world:setGravity(0.0, 9.8)
	self.world:setCallback(stageCollisionCallBack)
	-- Create players and their copters
	self.players = {}
	self.copters = {}
	for i = 1, self.PLAYER_COUNT do
		local player = Player:new(i)
		table.insert(self.players, player)
		table.insert(self.copters, Copter:new(self, player,  200 + (400 / (self.PLAYER_COUNT - 1)) * (i - 1), 300))
	end
	-- Misc
	self.emitters = {}
	self.hitTexts = {}
end

function Stage:update(dt)
	self.world:update(dt)
	for i, copter in pairs(self.copters) do
		copter:update(dt)
		if ((not self.finished) and (not copter:isDead())) then
			local x, y = copter:getX(), copter:getY()
			if (x < -20 or x > 820 or y < -20 or y > 620 or copter:getDamage() >= 200) then
				copter:die()
			end
		end
	end
	for i, emitter in pairs(self.emitters) do
		emitter:update(dt)
		if (emitter:isDead()) then
			table.remove(self.emitters, i)
		end
	end
	for i, hitText in pairs(self.hitTexts) do
		hitText:update(dt)
		if (hitText:isDead()) then
			table.remove(self.hitTexts, i)
		end
	end
	if (not self.finished) then
		-- Check if finished
		local alivePlayers = 0
		for i, copter in pairs(self.copters) do
			if (not copter:isDead()) then
				alivePlayers = alivePlayers + 1
				self.winner = i
			end
		end
		if (alivePlayers < 2) then
			self.finished = true
			if (alivePlayers == 0) then
				self.winner = 0
				love.audio.play(Resource:getSound("data/crowd_loose.wav"))
			else
				love.audio.play(Resource:getSound("data/crowd_win.wav"))
			end
		end
	elseif (self.winner > 0) then
		copter = self.copters[self.winner]
		local dx, dy = 400 - copter:getX(), 350 - copter:getY()
		copter:getBody():applyImpulse(dx * 0.1, dy * 0.1)
	end
end

function Stage:draw()
	for i, copter in pairs(self.copters) do
		copter:draw()
	end
	for i, emitter in pairs(self.emitters) do
		emitter:draw()
	end
	for i, hitText in pairs(self.hitTexts) do
		hitText:draw()
	end
	for i, copter in pairs(self.copters) do
		local x, y
		-- Tip
		if (not copter:isDead()) then
			x, y = math.floor(copter:getX()) + 16, math.floor(copter:getY()) - 48
			if (self.PLAYER_COLORS[i] ~= nil) then love.graphics.setColor(self.PLAYER_COLORS[i]) else love.graphics.setColor(255, 255, 255) end
			love.graphics.draw(self.tipImage, x, y)
			love.graphics.setColor(0, 0, 0)
			love.graphics.drawf("P" .. i, x - 16, y + 4, 32, love.align_center)
		end
		-- Score
		if (not self.finished) then
			local text
			if (copter:getDamage() < 200) then
				text = math.floor(100 - copter:getDamage() / 2) .. "%"
			else
				text = "Dead"
			end
			x = 200 - 50 + (400 / (self.PLAYER_COUNT - 1)) * (i - 1)
			if (self.PLAYER_COLORS[i] ~= nil) then love.graphics.setColor(self.PLAYER_COLORS[i]) else love.graphics.setColor(255, 255, 255) end
			love.graphics.drawf("P" .. i, x, 550, 100, love.align_center)
			love.graphics.drawf(text, x, 570, 100, love.align_center)
		end
	end
	if (self.finished) then
		local text
		if (self.winner > 0) then
			text = "WE'VE GOT A WINNER TODAY !"
		else
			text = "WHAT ?!"
		end
		love.graphics.setColor(255, 255, 255)
		love.graphics.drawf(text, 250, 250, 300, love.align_center)
	end
end

function Stage:ShowDamage(amount, x, y)
	table.insert(self.hitTexts, HitText:new(amount, x + math.random(-25, 25), y + math.random(-25, 25), self.DAMAGE_COLOR))
end

function Stage:Spark(x, y)
	table.insert(self.emitters, Emitter:new(x, y, Resource:getImage("data/fire.png"), 0.1, 200, 100, 0.2, 0.5, 1.0, true))
end

function Stage:Explode(x, y)
	table.insert(self.emitters, Emitter:new(x, y, Resource:getImage("data/fire.png"), 0.1, 1000, 100, 1.0, 2.0, 4.0, true))
	love.audio.play(Resource:getSound("data/explode.wav"))
end

function Stage:makeWind(user)
	local x, y = user:getX(), user:getY()
	for i, copter in pairs(self.copters) do
		if (copter ~= user) then
			local dx, dy = x - copter:getX(), y - copter:getY()
			local distance = math.sqrt(dx * dx + dy * dy)
			if (distance < 150) then
				local nx, ny = dx / distance, dy / distance
				copter:getBody():applyImpulse(nx * - 250.0, ny * - 250.0)
			end
		end
	end
	love.audio.play(Resource:getSound("data/wind.wav"))
end

function Stage:getScale()
	return self.SCALE
end

function Stage:getWorld()
	return self.world
end

function stageCollisionCallBack(first, second, contact)
	if (not Collidable:made(first)) then error("first element of a collision event isn't a Collidable") end
	if (not Collidable:made(second)) then error("second element of a collision event isn't a Collidable") end
	love.audio.play(Resource:getSound("data/wood_" .. math.random(1, 3) .. ".wav"))
	first:onCollision(second, contact)
	second:onCollision(first, contact)
end
